/*
 * $Id: DistanceData.java,v 1.2 2005/06/10 14:54:10 hartmut Exp $
 *
 * Copyright (C) 2004 by Brockmann Consult (info@brockmann-consult.de)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation. This program is distributed in the hope it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.bc.beam.visat.rangeFinder;

import org.esa.beam.framework.datamodel.GeoCoding;
import org.esa.beam.framework.datamodel.GeoPos;
import org.esa.beam.framework.datamodel.PixelPos;
import org.esa.beam.framework.dataop.maptransf.Ellipsoid;
import org.esa.beam.util.math.MathUtils;

import java.awt.*;

/**
 * A container for the data collected by the range finder tool.
 * @author Sabine Embacher
 */
class DistanceData {

    final static float MIN_EARTH_RADIUS = (float) Ellipsoid.WGS_84.getSemiMinor();
    final static float MAX_EARTH_RADIUS = (float) Ellipsoid.WGS_84.getSemiMajor();
    final static float MEAN_EARTH_RADIUS = 6371000;

    final static float MEAN_EARTH_RADIUS_KM = 0.001f * MEAN_EARTH_RADIUS;
    final static float MEAN_ERROR_FACTOR = MIN_EARTH_RADIUS / MAX_EARTH_RADIUS;

    final int xH;
    final int yH;
    final int xN;
    final int yN;
    final float lonH;
    final float latH;
    final float lonN;
    final float latN;
    final float lamH;
    final float phiH;
    final float lamN;
    final float phiN;
    final float distance;
    final float distanceError;

    public DistanceData(GeoCoding geoCoding, final Point pH, final Point pN) {
        this.xH = pH.x;
        this.yH = pH.y;
        this.xN = pN.x;
        this.yN = pN.y;
        final GeoPos geoPosH = geoCoding.getGeoPos(new PixelPos(xH, yH), null);
        final GeoPos geoPosN = geoCoding.getGeoPos(new PixelPos(xN, yN), null);
        this.lonH = geoPosH.getLon();
        this.latH = geoPosH.getLat();
        this.lonN = geoPosN.getLon();
        this.latN = geoPosN.getLat();
        this.lamH = (float) (MathUtils.DTOR * lonH);
        this.phiH = (float) (MathUtils.DTOR * latH);
        this.lamN = (float) (MathUtils.DTOR * lonN);
        this.phiN = (float) (MathUtils.DTOR * latN);
        this.distance = (float) (MEAN_EARTH_RADIUS_KM * Math.acos(Math.sin(phiH) * Math.sin(phiN) + Math.cos(phiH) * Math.cos(phiN) * Math.cos(Math.abs(lamN - lamH))));
        this.distanceError = distance * (1 - MEAN_ERROR_FACTOR);
    }
}
